#include <assert.h>
#include <math.h>
#include <stddef.h>

#include "blitz.h"
#include "tools.h"

BOOL blitz_entfernung_anhand_blende(double lz, double k, double* g) {

	BOOL params_ok = (lz > 0 && k > 0 && g != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*g = lz / k;

	return TRUE;
}

BOOL blitz_blende_anhand_entfernung(double lz, double g, double* k) {

	BOOL params_ok = (lz > 0 && g > 0 && k != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k = lz / g;

	return TRUE;
}

BOOL blitz_gesamt_leitzahl(double lz_1, double lz_2, double* lz_ges) {

	BOOL params_ok = (lz_1 > 0 && lz_2 > 0 && lz_ges != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*lz_ges = sqrt(tools_quad(lz_1) + tools_quad(lz_2));

	return TRUE;
}

BOOL blitz_gesamt_leitzahl_anhand_blenden(double k_1, double k_2, double* lz_ges) {

	BOOL params_ok = (k_1 > 0 && k_2 > 0 && lz_ges != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*lz_ges = sqrt(tools_quad(k_1) + tools_quad(k_2));

	return TRUE;
}

BOOL blitz_anzahl_offenblitz(double lz, double k, double g, double* anzahl) {

	BOOL params_ok = (lz > 0 && k > 0 && g > 0 && anzahl != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*anzahl = tools_quad(g * k / lz);

	return TRUE;
}

BOOL blitz_entfernung_nahaufnahme(double lz, double k, double beta, double* g) {

	BOOL params_ok = (lz > 0 && k > 0 && beta > 0 && g != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*g = lz / (k * (beta + 1));

	return TRUE;
}
