#include <assert.h>
#include <math.h>
#include <stddef.h>

#include "tools.h"
#include "stiefe.h"

BOOL stiefe_bereich(double f, double k, double z, double g, double* bereich) {

	double  hyp           = 0.0;
	BOOL    bis_unendlich = FALSE; /* Impliziert: Nenner <= 0 */
	BOOL    params_ok     = FALSE;

	stiefe_hyp(f, k, z, &hyp);

	bis_unendlich = (g >= hyp);
	params_ok     = (!bis_unendlich && f > 0 && k > 0 && z > 0 && g > f && bereich != NULL);
	params_ok     = params_ok && (tools_quad(f) != tools_quad(k * z * (g - f) / f)); /* f > 0 */

	assert(params_ok); if (!params_ok) return FALSE;

	*bereich = 2.0 * k * z * g * (g - f) / (tools_quad(f) - tools_quad(k * z * (g - f) / f));

	return TRUE;
}

BOOL stiefe_nahpunkt(double f, double k, double z, double g, double* nahpunkt) {

	BOOL params_ok = (f > 0 && k > 0 && z > 0 && g > f && nahpunkt != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*nahpunkt = tools_quad(f) * g / (tools_quad(f) + k * z * (g - f)); /* g > f  */

	return TRUE;
}

BOOL stiefe_fernpunkt(double f, double k, double z, double g, double* fernpunkt) {

	double  hyp           = 0.0;
	BOOL    bis_unendlich = FALSE; /* Impliziert Nenner <= 0 */
	BOOL    params_ok     = FALSE;

	stiefe_hyp(f, k, z, &hyp);

	bis_unendlich = (g >= hyp);

	params_ok = (!bis_unendlich && f > 0 && k > 0 && z > 0 && g > f && fernpunkt != NULL);
	params_ok = params_ok && (tools_quad(f) != k * z * (g - f));

	assert(params_ok); if (!params_ok) return FALSE;

	*fernpunkt = tools_quad(f) * g / (tools_quad(f) - k * z * (g - f));

	return TRUE;
}

BOOL stiefe_hyp(double f, double k, double z, double* hyp) {

	BOOL params_ok = (f > 0 && k > 0 && z > 0 && hyp != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*hyp = f * (1.0 + f / (k * z));

	return TRUE;
}

BOOL stiefe_bereich_abbm(double beta, double f, double k, double z, double* bereich) {

	BOOL params_ok = (beta > 0 && f > 0 && k > 0 && z > 0 && bereich != NULL);

	params_ok = params_ok && (tools_quad(f * beta) != tools_quad(k * z)); /*  Sonst ist Nenner 0 */

	assert(params_ok); if (!params_ok) return FALSE;

	*bereich = 2.0 * k * z * tools_quad(f) * (beta + 1.0) / (tools_quad(f * beta) - tools_quad(k * z));

	return TRUE;
}

BOOL stiefe_bereich_abbm_2(double beta, double k, double z, double* bereich) {

	BOOL params_ok = (beta > 0 && k > 0 && z > 0 && bereich != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*bereich = 2.0 * k * z * (beta + 1) / tools_quad(beta);

	return TRUE;
}

BOOL stiefe_nahpunkt_abbm(double beta, double f, double k, double z, double* nahpunkt) {

	BOOL params_ok = (beta > 0 && f > 0 && k > 0 && z > 0 && nahpunkt != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*nahpunkt = tools_quad(f) * (beta + 1.0) / (f * beta + k * z);

	return TRUE;
}

BOOL stiefe_fernpunkt_abbm(double beta, double f, double k, double z, double* fernpunkt) {

	BOOL nenner_ist_null = ((f * beta) == (k * z));
	BOOL params_ok       = (!nenner_ist_null && beta > 0 && f > 0 && k > 0 && z > 0 && fernpunkt != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*fernpunkt = tools_quad(f) * (beta + 1.0) / (f * beta - k * z);

	return TRUE;
}

BOOL stiefe_blende(double f, double g, double z, double t, double* k) {

	BOOL params_ok = (f > 0 && g > f && z > 0 && t > 0 && k != NULL);

	assert(params_ok); if (!params_ok) return FALSE;

	*k = tools_quad(f) * (sqrt(tools_quad(g) + tools_quad(t)) - g) / (t * z * (g - f)); /* g > f */

	return TRUE;
}
